/**
  * @file THERMO5_Click_PIC32.c
  * @brief This module is used to control a temperature
  *
  * @author Stefan Popovic
  *
  * @date 16/5/2017
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: P32MX795F512L
  * Dev.Board: EasyPIC Fusion v7
  * Oscillator: 80 MHz External
  * Ext. Modules: Thermostat Click Board
  * SW: MikroC PRO for PIC32 v3.6.1

  * NOTES:
    - Place THERMO5 click board on mikroBUS slot 1
    
  /*****************************************************************************
  * Module Preprocessor Constants
  *****************************************************************************/
  #define THERMO5_ADDR   0x4C << 1

  #define INT_DIODE      0x0029
  #define EXT_DIODE1     0x0110
  #define EXT_DIODE2     0x2324
  #define EXT_DIODE3     0x2A2B

  /*****************************************************************************
  * Module Preprocessor Macros
  *****************************************************************************/


  /*****************************************************************************
  * Module Typedefs
  *****************************************************************************/


  /*****************************************************************************
  * Includes
  *****************************************************************************/
  #include "built_in.h"
  #include "resources.h"
  #include <stdint.h>
  #include <stdbool.h>

  /*****************************************************************************
  * Module Variable Definitions
  *****************************************************************************/
  // TFT module connections
  char TFT_DataPort at LATE;
  sbit TFT_RST at LATD7_bit;
  sbit TFT_BLED at LATD2_bit;
  sbit TFT_RS at LATD9_bit;
  sbit TFT_CS at LATD10_bit;
  sbit TFT_RD at LATD5_bit;
  sbit TFT_WR at LATD4_bit;
  char TFT_DataPort_Direction at TRISE;
  sbit TFT_RST_Direction at TRISD7_bit;
  sbit TFT_BLED_Direction at TRISD2_bit;
  sbit TFT_RS_Direction at TRISD9_bit;
  sbit TFT_CS_Direction at TRISD10_bit;
  sbit TFT_RD_Direction at TRISD5_bit;
  sbit TFT_WR_Direction at TRISD4_bit;
  // End TFT module connections

 // Variables
  float diodeTemperature;
  char tmp_data[10];
  char temp[2];
  char result;
  char txt[15];
  void Temp_Read(void);

  void Set_TEMP_REF(unsigned short);
  void Set_TEMP_HYST(unsigned short);
  void Set_CONFIG(unsigned short);

  char data_;

  void DrawFrame()
   {
    TFT_Fill_Screen(CL_SILVER);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_SILVER, 0, 0, CL_SILVER, CL_SILVER);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("THERMO5 click", 80, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Internal Diode:", 10, 60 );
    TFT_Write_Text("External Diode 1:", 10, 100 );
    TFT_Write_Text("External Diode 2:", 10, 140 );
    TFT_Write_Text("External Diode 3:", 10, 180 );
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Set_Pen( CL_WHITE , 1 );
  }
  float TH5Read(unsigned int diode_reg)
    {
         unsigned error;
         unsigned int Temperature;
         float Temp_Output;
         char Diode_Reg_Low, Diode_Reg_High;
         char Temp_Low_Byte, Temp_High_Byte;

         Diode_Reg_Low = Lo(diode_reg);
         Diode_Reg_High = Hi(diode_reg);

         // Reading LSB
         tmp_data[0] = Diode_Reg_Low;
         
         I2C_Start();                           // I2C START condition
         error = I2C_Write(THERMO5_ADDR);
         error = I2C_Write(tmp_data[0]);
         I2C_Restart();                         // Repeated start
         error = I2C_Write(THERMO5_ADDR | 0x01);
         tmp_data[0] = I2C_Read(_I2C_ACK);
         tmp_data[1] = I2C_Read(_I2C_NACK);
         I2C_Stop();                             // I2C STOP condition
         
         Temp_Low_Byte = tmp_data[0];
         
         // Reading MSB
         tmp_data[0] = Diode_Reg_High;
         
         I2C_Start();                           // I2C START condition
         error = I2C_Write(THERMO5_ADDR);
         error = I2C_Write(tmp_data[0]);
          I2C_Restart();                         // Repeated start
           error = I2C_Write(THERMO5_ADDR | 0x01);
         tmp_data[0] = I2C_Read(_I2C_ACK);
         tmp_data[1] = I2C_Read(_I2C_NACK);
         I2C_Stop();                            // I2C STOP condition

         Temp_High_Byte = tmp_data[0];

        Temperature = (Temp_High_Byte << 8) | (Temp_Low_Byte);

        Temperature = Temperature >> 5;

        Temp_Output = Temperature * 0.125;

        return Temp_Output;
    }


  //Main program
  void main()
  {
   // MCU Init
   AD1PCFG = 0xFFFF;            // Configure AN pins as digital I/O
   JTAGEN_bit = 0;              // Disable JTAG
   TRISA  = 0;                  // Configure PORTA as output
   TRISB  = 0;                  // Configure PORTB as output
   TRISC  = 0;                  // Configure PORTC as output
   TRISD  = 0;                  // Configure PORTD as output

   LATA = 0;                    //Set PORTA value to zero
   LATB = 0;                    //Set PORTB value to zero
   LATC = 0;                    //Set PORTC value to zero
   LATD = 0;                    //Set PORTD value to zero

   // TFT Init
   TFT_BLED_Direction = 0;
   TFT_Init_ILI9341_8bit(320, 240);
   TFT_BLED = 1;
   DrawFrame();

   // I2C Init
   I2C2_Init_Advanced(50000, 100000);
   delay_ms(500);
   // Sets the I2C1 module active
   I2C_Set_Active(&I2C2_Start,
                  &I2C2_Restart,
                  &I2C2_Read,
                  &I2C2_Write,
                  &I2C2_Stop,
                  &I2C2_Is_Idle);

   diodeTemperature = 0.0;

   //***************************************************************************
   // Temperature read loop
   while (1)
   {
      TFT_Rectangle( 200, 50, 280, 200 );
      
      diodeTemperature = TH5Read(INT_DIODE);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 60 );

      diodeTemperature = TH5Read(EXT_DIODE1);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 100 );

      diodeTemperature = TH5Read(EXT_DIODE2);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 140 );

      diodeTemperature = TH5Read(EXT_DIODE3);
      sprintf( txt, "%.2f C", diodeTemperature);
      TFT_Write_Text( txt, 200, 180 );

      delay_ms(500);
   }
   //***************************************************************************
}